#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright © 2009 The Caffeine Developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import struct
import caffeinelogging as logging

FALL_BACK_FLV_LENGTH = 30 * 60

class FLVParsingError(Exception):
    def __init__(self, txt):
        Exception.__init__(self, txt)

def readSingleTypedValue(f):
    metadataValueType = struct.unpack('B', f.read(1))[0]
    if metadataValueType == 0:
        double = struct.unpack('>d', f.read(8))[0]
        return double
    elif metadataValueType == 1:
        f.read(1)
        return "BooleanValueIgnored"
    elif metadataValueType == 2:
        valueStrLen = struct.unpack('>H', f.read(2))[0]
        f.read(valueStrLen)
        return "StringValueIgnored"
    elif metadataValueType == 3:
        utfLen = 1
        while utfLen != 0:
            utfLen = struct.unpack('>H', f.read(2))[0]
            f.read(utfLen)
            ignore = readSingleTypedValue(f)
        return "ObjectIgnored"
    elif metadataValueType == 8:
        ignore = readMetadataArray(f)
        return "MixedArrayIgnored"
    elif metadataValueType == 9:
        return "ObjectEndIgnored"
    elif metadataValueType == 10:
        readSingleTypeArray(f)
        return "SingleTypeArrayIgnored"
    elif metadataValueType == 11:
        ignore = f.read(10)
        return "DateIgnored"
    else:
        raise FLVParsingError("This rudimentary parser can't handle this data type: " + str(metadataValueType))

def readSingleTypeArray(f):
    arraySize = struct.unpack('>I', f.read(4))[0]
    for i in range(0, arraySize):
        readSingleTypedValue(f)

def readMetadataArray(f):
    """Pass in a file handle to read an array of mixed types from the FLV file. Most
    data types are ignored, as we are only interested in 'duration' related metadata."""
    arraySize = struct.unpack('>I', f.read(4))[0]
    metadata = {}
    for i in range(0, arraySize):
        try:
            metadataNameLen = struct.unpack('>H', f.read(2))[0]
            metadataName = f.read(metadataNameLen)
            value = readSingleTypedValue(f)
            metadata[metadataName] = value
        except FLVParsingError, e:
            logging.warn("Error occurred while reading metadata (" + str(e) + "), but a suitable duration tag may have already been read in")
            return metadata
    return metadata

def parseFLVFile(flvFilePath):
    """Returns an integer, the number of seconds that the specified FLV video
    file will play for."""
    with open(flvFilePath, 'rb') as f:
        # Get the first three bytes, and ensure that they are as expected
        sig = f.read(3)
        if sig != 'FLV':
            raise FLVParsingError("'FLV' signature not found at the beginning of the file")

        # Read past some other junk we don't care about
        ignore = f.read(2)
        offset = struct.unpack('>I', f.read(4))[0]
        ignore = f.read(offset - f.tell())

        # Find the metadata tag that we want
        ignore = f.read(4)
        if struct.unpack('B', f.read(1))[0] != 18:
            raise FLVParsingError("Non-metadata tags found; cannot read")
        b1, b2, b3 = struct.unpack('3B', f.read(3))
        tagSize = (b1 << 16) + (b2 << 8) + b3
        ignore = f.read(7)
        namedatatype = struct.unpack('B', f.read(1))[0]
        if namedatatype != 2:
            raise FLVParsingError("Was expecting to find the tag name string; datatype was incorrect")
        len = struct.unpack('>H', f.read(2))[0]
        tagName = f.read(len)
        if tagName != "onMetaData":
            raise FLVParsingError("Did not find expected tag 'onMetaData'")
        valuedatatype = struct.unpack('B', f.read(1))[0]
        if valuedatatype != 8:
            raise FLVParsingError("The onMetaData tag was expected to be type 8 (an array)")
        
        metadata = readMetadataArray(f)
        return metadata

def getFLVLength(flvFilePath):
        # Read the array of metadata
        metadata = {}
        try:
            metadata = parseFLVFile(flvFilePath)
        except FLVParsingError, e:
            logging.warn("Was unable to parse the FLV file " + flvFilePath + "; will return a default duration of " + str(FALL_BACK_FLV_LENGTH) + " seconds")
            return FALL_BACK_FLV_LENGTH
        
        if metadata.has_key("duration"):
            return int(metadata["duration"])
        elif metadata.has_key("totalduration"):
            return int(metadata["totalduration"])
        else:
            logging.warn("No suitable duration tag was found in " + flvFilePath + "; will return a default duration of " + str(FALL_BACK_FLV_LENGTH) + " seconds")
            return FALL_BACK_FLV_LENGTH

def getProcessName(pid):
    """Gets process name from process id"""

    truncProcessName = open("/proc/%s/status" % pid).readline()[6:-1]
    processName = truncProcessName

    line = open("/proc/%s/cmdline" % pid).readline()
    parts = line.split("\x00")
    for part in parts:
        cmdName = part.split("/")[-1]

        if cmdName.startswith(truncProcessName):
            processName = cmdName

    return processName

def getProcesses():

    processList = []

    for pid in os.listdir("/proc/"):
        try:
            pid = int(pid)

        except:
            continue

        try:
            processName = getProcessName(pid).lower()

        except:
            continue

        processList.append((processName, pid))

    return processList

def isProcessRunning(name):

    for proc_name, pid in getProcesses():
        if name == proc_name:
            return True

    return False

